package bubble.resources.device;

import bubble.dao.device.FlexRouterDAO;
import bubble.model.account.Account;
import bubble.model.device.Device;
import bubble.model.device.FlexRouter;
import bubble.resources.account.AccountOwnedResource;
import bubble.service.device.DeviceService;
import lombok.extern.slf4j.Slf4j;
import org.glassfish.grizzly.http.server.Request;
import org.glassfish.jersey.server.ContainerRequest;
import org.springframework.beans.factory.annotation.Autowired;

import static bubble.service.device.FlexRouterService.pingFlexRouter;
import static org.cobbzilla.util.json.JsonUtil.COMPACT_MAPPER;
import static org.cobbzilla.util.json.JsonUtil.json;
import static org.cobbzilla.wizard.resources.ResourceUtil.invalidEx;
import static org.cobbzilla.wizard.resources.ResourceUtil.userPrincipal;

@Slf4j
public class FlexRoutersResource extends AccountOwnedResource<FlexRouter, FlexRouterDAO> {

    @Autowired private DeviceService deviceService;

    public FlexRoutersResource(Account account) { super(account); }

    @Override protected boolean isReadOnly(ContainerRequest ctx) {
        final Account caller = userPrincipal(ctx);
        return !caller.admin();
    }

    @Override protected FlexRouter setReferences(ContainerRequest ctx, Request req, Account caller, FlexRouter router) {

        if (!router.hasProxyPort()) throw invalidEx("err.proxyPort.required");
        router.setPort(router.getProxy_port());

        final String ip = router.getIp();
        final Device device = deviceService.findDeviceByIp(ip);
        if (device == null) throw invalidEx("err.device.notFound");

        if (pingFlexRouter(router).inactive()) {
            log.info("setReferences: router not active: "+json(router, COMPACT_MAPPER));
            throw invalidEx("err.token.required");
        }

        if (!router.hasAuthToken()) throw invalidEx("err.token.required");
        router.setToken(router.getAuth_token());

        log.info("setReferences: FlexRouter=\n"+json(router)+"\n");
        return super.setReferences(ctx, req, caller, router);
    }

}
