package bubble.service.device;

import bubble.dao.device.FlexRouterDAO;
import bubble.model.device.FlexRouter;
import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.cobbzilla.util.daemon.SimpleDaemon;
import org.cobbzilla.util.http.HttpRequestBean;
import org.cobbzilla.util.http.HttpResponseBean;
import org.cobbzilla.util.http.HttpUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

import static java.util.concurrent.TimeUnit.MINUTES;
import static org.cobbzilla.util.daemon.ZillaRuntime.shortError;
import static org.cobbzilla.util.http.HttpMethods.POST;

@Service @Slf4j
public class FlexRouterService extends SimpleDaemon {

    @Getter private final long sleepTime = MINUTES.toMillis(2);

    @Autowired private FlexRouterDAO flexRouterDAO;

    @Override protected void process() {
        try {
            final List<FlexRouter> routers = flexRouterDAO.findEnabledAndRegistered();
            for (FlexRouter router : routers) {
                FlexRouter update = pingFlexRouter(router);
                if (update != null) {
                    flexRouterDAO.update(update);
                }
            }
        } catch (Exception e) {
            log.error("process: "+shortError(e));
        }
    }

    public static FlexRouter pingFlexRouter(FlexRouter router) {
        final String prefix = "pingRouter(" + router + "): ";
        final HttpRequestBean request = new HttpRequestBean(POST, router.pingUrl(), router.pingObject());
        try {
            final HttpResponseBean response = HttpUtil.getResponse(request);
            if (!response.isOk()) {
                log.error(prefix+"response not OK, marking inactive: "+response);
                return router.setActive(false);
            }
            if (log.isInfoEnabled()) log.info(prefix+"router is ok, marking active and updating lastSeen");
            return router.setActive(true).setLastSeen();

        } catch (Exception e) {
            log.error(prefix+"error (marking inactive): "+shortError(e));
            return router.setActive(false);
        }
    }

}
